///<reference path="../../../.d.ts"/>
"use strict";
var os = require("os");
var AndroidApplicationManager = (function () {
    function AndroidApplicationManager(adb, identifier, $options, $staticConfig) {
        this.adb = adb;
        this.identifier = identifier;
        this.$options = $options;
        this.$staticConfig = $staticConfig;
    }
    AndroidApplicationManager.prototype.getInstalledApplications = function () {
        var _this = this;
        return (function () {
            if (!_this._installedApplications) {
                var result = _this.adb.executeShellCommand("pm list packages").wait();
                var regex = /package:(.+)/;
                _this._installedApplications = _.map(result.split(os.EOL), function (packageString) {
                    var match = packageString.match(regex);
                    return match ? match[1] : null;
                }).filter(function (parsedPackage) { return parsedPackage != null; });
            }
            return _this._installedApplications;
        }).future()();
    };
    AndroidApplicationManager.prototype.installApplication = function (packageFilePath) {
        return this.adb.executeCommand("install -r \"" + packageFilePath + "\"");
    };
    AndroidApplicationManager.prototype.uninstallApplication = function (appIdentifier) {
        return this.adb.executeShellCommand("pm uninstall \"" + appIdentifier + "\"");
    };
    AndroidApplicationManager.prototype.startApplication = function (appIdentifier) {
        return this.adb.executeShellCommand("am start -a android.intent.action.MAIN -n " + appIdentifier + "/" + this.$staticConfig.START_PACKAGE_ACTIVITY_NAME + " -c android.intent.category.LAUNCHER");
    };
    AndroidApplicationManager.prototype.stopApplication = function (appIdentifier) {
        return this.adb.executeShellCommand("am force-stop " + appIdentifier);
    };
    AndroidApplicationManager.prototype.restartApplication = function (appIdentifier) {
        var _this = this;
        return (function () {
            _this.stopApplication(appIdentifier).wait();
            _this.startApplication(appIdentifier).wait();
        }).future()();
    };
    return AndroidApplicationManager;
})();
exports.AndroidApplicationManager = AndroidApplicationManager;
