///<reference path="../../../.d.ts"/>
"use strict";
var util = require("util");
var AndroidDebugBridge = (function () {
    function AndroidDebugBridge(identifier, $childProcess, $errors, $logger, $staticConfig) {
        this.identifier = identifier;
        this.$childProcess = $childProcess;
        this.$errors = $errors;
        this.$logger = $logger;
        this.$staticConfig = $staticConfig;
    }
    AndroidDebugBridge.prototype.executeCommand = function () {
        var _this = this;
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i - 0] = arguments[_i];
        }
        return (function () {
            var command = _this.composeCommand(args).wait();
            return _this.$childProcess.exec(command).wait();
        }).future()();
    };
    AndroidDebugBridge.prototype.executeShellCommand = function () {
        var _this = this;
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i - 0] = arguments[_i];
        }
        return (function () {
            args.unshift("shell");
            var shellCommand = _this.composeCommand(args).wait();
            _this.$logger.trace("Shell command " + shellCommand);
            return _this.$childProcess.exec(shellCommand).wait();
        }).future()();
    };
    AndroidDebugBridge.prototype.sendBroadcastToDevice = function (action, extras) {
        var _this = this;
        if (extras === void 0) { extras = {}; }
        return (function () {
            var broadcastCommand = "am broadcast -a \"" + action + "\"";
            _.each(extras, function (value, key) { return broadcastCommand += " -e \"" + key + "\" \"" + value + "\""; });
            var result = _this.executeShellCommand(broadcastCommand).wait();
            _this.$logger.trace("Broadcast result " + result + " from " + broadcastCommand);
            var match = result.match(/Broadcast completed: result=(\d+)/);
            if (match) {
                return +match[1];
            }
            _this.$errors.failWithoutHelp("Unable to broadcast to android device:\n%s", result);
        }).future()();
    };
    AndroidDebugBridge.prototype.composeCommand = function (args) {
        var _this = this;
        return (function () {
            var command = util.format.apply(null, args);
            var result = "\"" + _this.$staticConfig.getAdbFilePath().wait() + "\" -s " + _this.identifier + " " + command;
            return result;
        }).future()();
    };
    return AndroidDebugBridge;
})();
exports.AndroidDebugBridge = AndroidDebugBridge;
