///<reference path="./../../../.d.ts"/>
"use strict";
var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
var ref = require("ref");
var os = require("os");
var IOSDevice = require("./../ios/ios-device");
var AndroidDevice = require("./../android/android-device");
var CoreTypes = require("./../ios/ios-core");
var Signal = require("./../../events/signal");
var Future = require("fibers/future");
var helpers = require("./../../helpers");
var DeviceDiscovery = (function () {
    function DeviceDiscovery() {
        this.devices = {};
        this.deviceFound = null;
        this.deviceLost = null;
        this.deviceFound = new Signal.Signal();
        this.deviceLost = new Signal.Signal();
    }
    DeviceDiscovery.prototype.addDevice = function (device) {
        this.devices[device.deviceInfo.identifier] = device;
        this.raiseOnDeviceFound(device);
    };
    DeviceDiscovery.prototype.removeDevice = function (deviceIdentifier) {
        var device = this.devices[deviceIdentifier];
        if (!device) {
            return;
        }
        delete this.devices[deviceIdentifier];
        this.raiseOnDeviceLost(device);
    };
    DeviceDiscovery.prototype.startLookingForDevices = function () {
        return undefined;
    };
    DeviceDiscovery.prototype.raiseOnDeviceFound = function (device) {
        this.deviceFound.dispatch(device);
    };
    DeviceDiscovery.prototype.raiseOnDeviceLost = function (device) {
        this.deviceLost.dispatch(device);
    };
    return DeviceDiscovery;
})();
exports.DeviceDiscovery = DeviceDiscovery;
$injector.register("deviceDiscovery", DeviceDiscovery);
var IOSDeviceDiscovery = (function (_super) {
    __extends(IOSDeviceDiscovery, _super);
    function IOSDeviceDiscovery($coreFoundation, $mobileDevice, $errors, $injector, $options, $utils) {
        _super.call(this);
        this.$coreFoundation = $coreFoundation;
        this.$mobileDevice = $mobileDevice;
        this.$errors = $errors;
        this.$injector = $injector;
        this.$options = $options;
        this.$utils = $utils;
        this.timerCallbackPtr = null;
        this.notificationCallbackPtr = null;
        this.timerCallbackPtr = CoreTypes.CoreTypes.cf_run_loop_timer_callback.toPointer(IOSDeviceDiscovery.timerCallback);
        this.notificationCallbackPtr = CoreTypes.CoreTypes.am_device_notification_callback.toPointer(IOSDeviceDiscovery.deviceNotificationCallback);
    }
    IOSDeviceDiscovery.prototype.startLookingForDevices = function () {
        var _this = this;
        return (function () {
            _this.subscribeForNotifications();
            var defaultTimeoutInSeconds = 1;
            var parsedTimeout = _this.$utils.getParsedTimeout(1);
            var timeout = parsedTimeout > defaultTimeoutInSeconds ? parsedTimeout / 1000 : defaultTimeoutInSeconds;
            _this.startRunLoopWithTimer(timeout);
        }).future()();
    };
    IOSDeviceDiscovery.deviceNotificationCallback = function (devicePointer, user) {
        var iOSDeviceDiscovery = $injector.resolve("iOSDeviceDiscovery");
        var deviceInfo = ref.deref(devicePointer);
        if (deviceInfo.msg === IOSDeviceDiscovery.ADNCI_MSG_CONNECTED) {
            iOSDeviceDiscovery.createAndAddDevice(deviceInfo.dev);
        }
        else if (deviceInfo.msg === IOSDeviceDiscovery.ADNCI_MSG_DISCONNECTED) {
            var deviceIdentifier = iOSDeviceDiscovery.$coreFoundation.convertCFStringToCString(iOSDeviceDiscovery.$mobileDevice.deviceCopyDeviceIdentifier(deviceInfo.dev));
            iOSDeviceDiscovery.removeDevice(deviceIdentifier);
        }
    };
    IOSDeviceDiscovery.timerCallback = function () {
        var iOSDeviceDiscovery = $injector.resolve("iOSDeviceDiscovery");
        iOSDeviceDiscovery.$coreFoundation.runLoopStop(iOSDeviceDiscovery.$coreFoundation.runLoopGetCurrent());
    };
    IOSDeviceDiscovery.prototype.validateResult = function (result, error) {
        if (result !== 0) {
            this.$errors.fail(error);
        }
    };
    IOSDeviceDiscovery.prototype.subscribeForNotifications = function () {
        var notifyFunction = ref.alloc(CoreTypes.CoreTypes.amDeviceNotificationRef);
        var result = this.$mobileDevice.deviceNotificationSubscribe(this.notificationCallbackPtr, 0, 0, 0, notifyFunction);
        var error = IOSDeviceDiscovery.APPLE_SERVICE_NOT_STARTED_ERROR_CODE ?
            "Cannot run and complete operations on iOS devices because Apple Mobile Device Service is not started. Verify that iTunes is installed and running on your system." : "Unable to subscribe for notifications";
        this.validateResult(result, error);
        this.$errors.verifyHeap("subscribeForNotifications");
    };
    IOSDeviceDiscovery.prototype.startRunLoopWithTimer = function (timeout) {
        var kCFRunLoopDefaultMode = this.$coreFoundation.kCFRunLoopDefaultMode();
        var timer = null;
        if (timeout > 0) {
            var currentTime = this.$coreFoundation.absoluteTimeGetCurrent() + timeout;
            timer = this.$coreFoundation.runLoopTimerCreate(null, currentTime, 0, 0, 0, this.timerCallbackPtr, null);
            this.$coreFoundation.runLoopAddTimer(this.$coreFoundation.runLoopGetCurrent(), timer, kCFRunLoopDefaultMode);
        }
        this.$coreFoundation.runLoopRun();
        if (timeout > 0) {
            this.$coreFoundation.runLoopRemoveTimer(this.$coreFoundation.runLoopGetCurrent(), timer, kCFRunLoopDefaultMode);
        }
        this.$errors.verifyHeap("startRunLoopWithTimer");
    };
    IOSDeviceDiscovery.prototype.createAndAddDevice = function (devicePointer) {
        var device = this.$injector.resolve(IOSDevice.IOSDevice, { devicePointer: devicePointer });
        this.addDevice(device);
    };
    IOSDeviceDiscovery.ADNCI_MSG_CONNECTED = 1;
    IOSDeviceDiscovery.ADNCI_MSG_DISCONNECTED = 2;
    IOSDeviceDiscovery.APPLE_SERVICE_NOT_STARTED_ERROR_CODE = 0xE8000063;
    return IOSDeviceDiscovery;
})(DeviceDiscovery);
var IOSDeviceDiscoveryStub = (function (_super) {
    __extends(IOSDeviceDiscoveryStub, _super);
    function IOSDeviceDiscoveryStub($logger, $staticConfig, $hostInfo, error) {
        _super.call(this);
        this.$logger = $logger;
        this.$staticConfig = $staticConfig;
        this.$hostInfo = $hostInfo;
        this.error = error;
    }
    IOSDeviceDiscoveryStub.prototype.startLookingForDevices = function () {
        if (this.error) {
            this.$logger.warn(this.error);
        }
        else if (this.$hostInfo.isLinux) {
            this.$logger.warn("In this version of the %s command-line interface, you cannot use connected iOS devices.", this.$staticConfig.CLIENT_NAME.toLowerCase());
        }
        return Future.fromResult();
    };
    return IOSDeviceDiscoveryStub;
})(DeviceDiscovery);
$injector.register("iOSDeviceDiscovery", function ($errors, $logger, $fs, $injector, $iTunesValidator, $staticConfig, $hostInfo) {
    var error = $iTunesValidator.getError().wait();
    var result = null;
    if (error || $hostInfo.isLinux) {
        result = new IOSDeviceDiscoveryStub($logger, $staticConfig, $hostInfo, error);
    }
    else {
        result = $injector.resolve(IOSDeviceDiscovery);
    }
    return result;
});
var AndroidDeviceDiscovery = (function (_super) {
    __extends(AndroidDeviceDiscovery, _super);
    function AndroidDeviceDiscovery($childProcess, $injector, $staticConfig) {
        _super.call(this);
        this.$childProcess = $childProcess;
        this.$injector = $injector;
        this.$staticConfig = $staticConfig;
    }
    AndroidDeviceDiscovery.prototype.createAndAddDevice = function (deviceIdentifier) {
        var _this = this;
        return (function () {
            var device = _this.$injector.resolve(AndroidDevice.AndroidDevice, { identifier: deviceIdentifier });
            _this.addDevice(device);
        }).future()();
    };
    AndroidDeviceDiscovery.prototype.startLookingForDevices = function () {
        var _this = this;
        return (function () {
            _this.ensureAdbServerStarted().wait();
            var requestAllDevicesCommand = "\"" + _this.$staticConfig.getAdbFilePath().wait() + "\" devices";
            var result = _this.$childProcess.exec(requestAllDevicesCommand).wait();
            var devices = result.toString().split(os.EOL).slice(1)
                .filter(function (element) { return !helpers.isNullOrWhitespace(element); })
                .map(function (element) {
                var parts = element.split("\t");
                var identifier = parts[0];
                var state = parts[1];
                if (state === "device") {
                    _this.createAndAddDevice(identifier).wait();
                }
            });
        }).future()();
    };
    AndroidDeviceDiscovery.prototype.ensureAdbServerStarted = function () {
        var startAdbServerCommand = "\"" + this.$staticConfig.getAdbFilePath().wait() + "\" start-server";
        return this.$childProcess.exec(startAdbServerCommand);
    };
    return AndroidDeviceDiscovery;
})(DeviceDiscovery);
exports.AndroidDeviceDiscovery = AndroidDeviceDiscovery;
$injector.register("androidDeviceDiscovery", AndroidDeviceDiscovery);
