///<reference path="../.d.ts"/>
"use strict";
var os = require("os");
var child_process = require("child_process");
var osenv = require("osenv");
var Future = require("fibers/future");
var SysInfo = (function () {
    function SysInfo($childProcess, $iTunesValidator, $logger, $hostInfo) {
        this.$childProcess = $childProcess;
        this.$iTunesValidator = $iTunesValidator;
        this.$logger = $logger;
        this.$hostInfo = $hostInfo;
        this.sysInfoCache = undefined;
    }
    SysInfo.prototype.getSysInfo = function () {
        if (!this.sysInfoCache) {
            var res = Object.create(null);
            var procOutput;
            var packageJson = require("../../package.json");
            res.procInfo = packageJson.name + "/" + packageJson.version;
            res.platform = os.platform();
            res.os = this.$hostInfo.isWindows ? this.winVer() : this.unixVer();
            res.shell = osenv.shell();
            try {
                res.dotNetVer = this.$hostInfo.dotNetVersion().wait();
            }
            catch (err) {
                res.dotNetVer = ".Net is not installed.";
            }
            res.procArch = process.arch;
            res.nodeVer = process.version;
            procOutput = this.$childProcess.exec("npm -v").wait();
            res.npmVer = procOutput ? procOutput.split("\n")[0] : null;
            try {
                var output = this.$childProcess.spawnFromEvent("java", ["-version"], "exit").wait().stderr.split(os.EOL)[0];
                res.javaVer = /^java version \"((?:\d+\.)+(?:\d+))/i.exec(output)[1];
            }
            catch (e) {
                res.javaVer = null;
            }
            procOutput = this.exec("ant -version");
            res.antVer = procOutput ? procOutput.split(os.EOL)[0] : null;
            res.nodeGypVer = this.exec("node-gyp -v");
            res.xcodeVer = this.$hostInfo.isDarwin ? this.exec("xcodebuild -version") : null;
            res.itunesInstalled = this.$iTunesValidator.getError().wait() === null;
            procOutput = this.exec("adb version");
            res.adbVer = procOutput ? procOutput.split(os.EOL)[0] : null;
            procOutput = this.execAndroidH();
            res.androidInstalled = procOutput ? _.contains(procOutput, "android") : false;
            procOutput = this.exec("mono --version");
            if (!!procOutput) {
                var match = SysInfo.monoVerRegExp.exec(procOutput);
                res.monoVer = match ? match[1] : null;
            }
            else {
                res.monoVer = null;
            }
            this.sysInfoCache = res;
        }
        return this.sysInfoCache;
    };
    SysInfo.prototype.exec = function (cmd) {
        try {
            return this.$childProcess.exec(cmd).wait();
        }
        catch (e) {
            return null;
        }
    };
    SysInfo.prototype.execAndroidH = function () {
        var _this = this;
        var future = new Future();
        var callback = function (error, stdout, stderr) {
            _this.$logger.trace("Exec android -h \n stdout: %s \n stderr: %s", stdout.toString(), stderr.toString());
            var err = error;
            if (error && err.code !== 1 && !err.killed && !err.signal) {
                future.return(null);
            }
            else {
                future.return(stdout);
            }
        };
        child_process.exec("android -h", callback);
        var result = future.wait();
        return result;
    };
    SysInfo.prototype.winVer = function () {
        return this.readRegistryValue("ProductName").wait() + " " +
            this.readRegistryValue("CurrentVersion").wait() + "." +
            this.readRegistryValue("CurrentBuild").wait();
    };
    SysInfo.prototype.readRegistryValue = function (valueName) {
        var future = new Future();
        var Winreg = require("winreg");
        var regKey = new Winreg({
            hive: Winreg.HKLM,
            key: '\\Software\\Microsoft\\Windows NT\\CurrentVersion'
        });
        regKey.get(valueName, function (err, value) {
            if (err) {
                future.throw(err);
            }
            else {
                future.return(value.value);
            }
        });
        return future;
    };
    SysInfo.prototype.unixVer = function () {
        return this.$childProcess.exec("uname -a").wait();
    };
    SysInfo.monoVerRegExp = /version (\d+[.]\d+[.]\d+) /gm;
    return SysInfo;
})();
exports.SysInfo = SysInfo;
$injector.register("sysInfo", SysInfo);
