///<reference path=".d.ts"/>
"use strict";
var Future = require("fibers/future");
var npm = require("npm");
var NodePackageManager = (function () {
    function NodePackageManager($logger, $errors, $fs, $lockfile, $options) {
        this.$logger = $logger;
        this.$errors = $errors;
        this.$fs = $fs;
        this.$lockfile = $lockfile;
        this.$options = $options;
    }
    NodePackageManager.prototype.getCache = function () {
        return npm.cache;
    };
    NodePackageManager.prototype.load = function (config) {
        var future = new Future();
        npm.load(config, function (err) {
            if (err) {
                future.throw(err);
            }
            else {
                future.return();
            }
        });
        return future;
    };
    NodePackageManager.prototype.install = function (packageName, pathToSave, config) {
        if (this.$options.ignoreScripts) {
            config = config || {};
            config["ignore-scripts"] = true;
        }
        return this.loadAndExecute("install", [pathToSave, packageName], { config: config });
    };
    NodePackageManager.prototype.uninstall = function (packageName, config) {
        return this.loadAndExecute("uninstall", [[packageName]], { config: config });
    };
    NodePackageManager.prototype.cache = function (packageName, version, config) {
        return this.loadAndExecute("cache", [packageName, version, undefined, false], { subCommandName: "add", config: config });
    };
    NodePackageManager.prototype.cacheUnpack = function (packageName, version, unpackTarget) {
        return this.loadAndExecute("cache", [packageName, version, unpackTarget, null, null, null, null], { subCommandName: "unpack" });
    };
    NodePackageManager.prototype.view = function (packageName, propertyName) {
        return this.loadAndExecute("view", [[packageName, propertyName], [false]]);
    };
    NodePackageManager.prototype.loadAndExecute = function (commandName, args, opts) {
        var _this = this;
        return (function () {
            opts = opts || {};
            _this.load(opts.config).wait();
            return _this.executeCore(commandName, args, opts.subCommandName).wait();
        }).future()();
    };
    NodePackageManager.prototype.executeCore = function (commandName, args, subCommandName) {
        var future = new Future();
        var callback = function (err, data) {
            if (err) {
                future.throw(err);
            }
            else {
                future.return(data);
            }
        };
        args.push(callback);
        var command = subCommandName ? npm.commands[commandName][subCommandName] : npm.commands[commandName];
        command.apply(this, args);
        return future;
    };
    return NodePackageManager;
})();
exports.NodePackageManager = NodePackageManager;
$injector.register("npm", NodePackageManager);
