///<reference path="../.d.ts"/>
"use strict";
var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
var path = require("path");
var shell = require("shelljs");
var util = require("util");
var Future = require("fibers/future");
var constants = require("../constants");
var semver = require("semver");
var androidProjectPropertiesManagerLib = require("./android-project-properties-manager");
var projectServiceBaseLib = require("./platform-project-service-base");
var AndroidProjectService = (function (_super) {
    __extends(AndroidProjectService, _super);
    function AndroidProjectService($androidEmulatorServices, $childProcess, $errors, $hostInfo, $injector, $logger, $options, $projectData, $propertiesParser, $sysInfo, $fs) {
        _super.call(this, $fs);
        this.$androidEmulatorServices = $androidEmulatorServices;
        this.$childProcess = $childProcess;
        this.$errors = $errors;
        this.$hostInfo = $hostInfo;
        this.$injector = $injector;
        this.$logger = $logger;
        this.$options = $options;
        this.$projectData = $projectData;
        this.$propertiesParser = $propertiesParser;
        this.$sysInfo = $sysInfo;
        this.SUPPORTED_TARGETS = ["android-17", "android-18", "android-19", "android-21", "android-22"];
        this._platformData = null;
        this.installedTargetsCache = null;
        this._androidProjectPropertiesManagers = Object.create(null);
    }
    Object.defineProperty(AndroidProjectService.prototype, "platformData", {
        get: function () {
            if (!this._platformData) {
                var projectRoot = path.join(this.$projectData.platformsDir, "android");
                this._platformData = {
                    frameworkPackageName: "tns-android",
                    normalizedPlatformName: "Android",
                    appDestinationDirectoryPath: path.join(projectRoot, "assets"),
                    appResourcesDestinationDirectoryPath: path.join(projectRoot, "res"),
                    platformProjectService: this,
                    emulatorServices: this.$androidEmulatorServices,
                    projectRoot: projectRoot,
                    deviceBuildOutputPath: path.join(this.$projectData.platformsDir, "android", "bin"),
                    validPackageNamesForDevice: [
                        util.format("%s-%s.%s", this.$projectData.projectName, "debug", "apk"),
                        util.format("%s-%s.%s", this.$projectData.projectName, "release", "apk")
                    ],
                    frameworkFilesExtensions: [".jar", ".dat", ".so"],
                    configurationFileName: "AndroidManifest.xml",
                    configurationFilePath: path.join(this.$projectData.platformsDir, "android", "AndroidManifest.xml"),
                    mergeXmlConfig: [{ "nodename": "manifest", "attrname": "*" }, { "nodename": "application", "attrname": "*" }]
                };
            }
            return this._platformData;
        },
        enumerable: true,
        configurable: true
    });
    AndroidProjectService.prototype.validate = function () {
        var _this = this;
        return (function () {
            _this.validatePackageName(_this.$projectData.projectId);
            _this.validateProjectName(_this.$projectData.projectName);
            _this.checkJava().wait() && _this.checkAnt().wait() && _this.checkAndroid().wait();
        }).future()();
    };
    AndroidProjectService.prototype.createProject = function (projectRoot, frameworkDir) {
        var _this = this;
        return (function () {
            _this.$fs.ensureDirectoryExists(projectRoot).wait();
            var newTarget = _this.getAndroidTarget(frameworkDir).wait();
            _this.$logger.trace("Using Android SDK '" + newTarget + "'.");
            var versionNumber = _.last(newTarget.split("-"));
            if (_this.$options.symlink) {
                _this.copyResValues(projectRoot, frameworkDir, versionNumber).wait();
                _this.copy(projectRoot, frameworkDir, ".project AndroidManifest.xml project.properties custom_rules.xml", "-f").wait();
                _this.symlinkDirectory("assets", projectRoot, frameworkDir).wait();
                _this.symlinkDirectory("libs", projectRoot, frameworkDir).wait();
            }
            else {
                _this.copyResValues(projectRoot, frameworkDir, versionNumber).wait();
                _this.copy(projectRoot, frameworkDir, "assets libs", "-R").wait();
                _this.copy(projectRoot, frameworkDir, ".project AndroidManifest.xml project.properties custom_rules.xml", "-f").wait();
            }
            if (newTarget) {
                _this.updateTarget(projectRoot, newTarget).wait();
            }
            var packageName = _this.$projectData.projectId;
            var packageAsPath = packageName.replace(/\./g, path.sep);
            var activityDir = path.join(projectRoot, 'src', packageAsPath);
            _this.$fs.createDirectory(activityDir).wait();
        }).future()();
    };
    AndroidProjectService.prototype.copyResValues = function (projectRoot, frameworkDir, versionNumber) {
        var _this = this;
        return (function () {
            var resSourceDir = path.join(frameworkDir, AndroidProjectService.RES_DIRNAME);
            var resDestinationDir = path.join(projectRoot, AndroidProjectService.RES_DIRNAME);
            _this.$fs.createDirectory(resDestinationDir).wait();
            var versionDirName = AndroidProjectService.VALUES_VERSION_DIRNAME_PREFIX + versionNumber;
            var directoriesToCopy = [AndroidProjectService.VALUES_DIRNAME];
            var directoriesInResFolder = _this.$fs.readDirectory(resSourceDir).wait();
            var integerFrameworkVersion = parseInt(versionNumber);
            var versionDir = _.find(directoriesInResFolder, function (dir) { return dir === versionDirName; }) ||
                _(directoriesInResFolder)
                    .map(function (dir) {
                    return {
                        dirName: dir,
                        sdkNum: parseInt(dir.substr(AndroidProjectService.VALUES_VERSION_DIRNAME_PREFIX.length))
                    };
                })
                    .filter(function (dir) { return dir.dirName.match(AndroidProjectService.VALUES_VERSION_DIRNAME_PREFIX) && dir.sdkNum && (!integerFrameworkVersion || (integerFrameworkVersion >= dir.sdkNum)); })
                    .max(function (dir) { return dir.sdkNum; })
                    .dirName;
            if (versionDir) {
                directoriesToCopy.push(versionDir);
            }
            _this.copy(resDestinationDir, resSourceDir, directoriesToCopy.join(" "), "-R").wait();
        }).future()();
    };
    AndroidProjectService.prototype.interpolateData = function (projectRoot) {
        var _this = this;
        return (function () {
            var manifestPath = path.join(projectRoot, "AndroidManifest.xml");
            var safeActivityName = _this.$projectData.projectName.replace(/\W/g, '');
            shell.sed('-i', /__PACKAGE__/, _this.$projectData.projectId, manifestPath);
            shell.sed('-i', /__APILEVEL__/, _this.getTarget(projectRoot).wait().split('-')[1], manifestPath);
            var stringsFilePath = path.join(projectRoot, 'res', 'values', 'strings.xml');
            shell.sed('-i', /__NAME__/, _this.$projectData.projectName, stringsFilePath);
            shell.sed('-i', /__TITLE_ACTIVITY__/, _this.$projectData.projectName, stringsFilePath);
            shell.sed('-i', /__NAME__/, _this.$projectData.projectName, path.join(projectRoot, '.project'));
        }).future()();
    };
    AndroidProjectService.prototype.afterCreateProject = function (projectRoot) {
        var _this = this;
        return (function () {
            var targetApi = _this.getTarget(projectRoot).wait();
            _this.$logger.trace("Android target: %s", targetApi);
            _this.runAndroidUpdate(projectRoot, targetApi).wait();
            _this.adjustMinSdk(projectRoot);
        }).future()();
    };
    AndroidProjectService.prototype.adjustMinSdk = function (projectRoot) {
        var manifestPath = path.join(projectRoot, "AndroidManifest.xml");
        var apiLevel = this.getTarget(projectRoot).wait().split('-')[1];
        if (apiLevel === "MNC") {
            shell.sed('-i', /android:minSdkVersion=".*?"/, "android:minSdkVersion=\"" + apiLevel + "\"", manifestPath);
        }
    };
    AndroidProjectService.prototype.getDebugOnDeviceSetup = function () {
        return {};
    };
    AndroidProjectService.prototype.canUpdatePlatform = function (currentVersion, newVersion) {
        return Future.fromResult(true);
    };
    AndroidProjectService.prototype.updatePlatform = function (currentVersion, newVersion) {
        return (function () { }).future()();
    };
    AndroidProjectService.prototype.buildProject = function (projectRoot) {
        var buildConfiguration = this.$options.release ? "release" : "debug";
        var args = this.getAntArgs(buildConfiguration, projectRoot);
        return this.spawn('ant', args);
    };
    AndroidProjectService.prototype.isPlatformPrepared = function (projectRoot) {
        return this.$fs.exists(path.join(projectRoot, "assets", constants.APP_FOLDER_NAME));
    };
    AndroidProjectService.prototype.getProjectPropertiesManager = function (filePath) {
        if (!this._androidProjectPropertiesManagers[filePath]) {
            this._androidProjectPropertiesManagers[filePath] = this.$injector.resolve(androidProjectPropertiesManagerLib.AndroidProjectPropertiesManager, { directoryPath: filePath });
        }
        return this._androidProjectPropertiesManagers[filePath];
    };
    AndroidProjectService.prototype.parseProjectProperties = function (projDir, destDir) {
        var _this = this;
        return (function () {
            projDir = projDir.trim();
            var projProp = path.join(projDir, "project.properties");
            if (!_this.$fs.exists(projProp).wait()) {
                _this.$logger.warn("Warning: File %s does not exist", projProp);
                return;
            }
            var projectPropertiesManager = _this.getProjectPropertiesManager(projDir);
            var references = projectPropertiesManager.getProjectReferences().wait();
            _.each(references, function (reference) {
                var adjustedPath = _this.$fs.isRelativePath(reference.path) ? path.join(projDir, reference.path) : reference.path;
                _this.parseProjectProperties(adjustedPath, destDir).wait();
            });
            _this.$logger.info("Copying %s", projDir);
            shell.cp("-Rf", projDir, destDir);
            var targetDir = path.join(destDir, path.basename(projDir));
            var targetSdk = "android-" + (_this.$options.sdk || AndroidProjectService.MIN_SUPPORTED_VERSION);
            _this.$logger.info("Generate build.xml for %s", targetDir);
            _this.runAndroidUpdate(targetDir, targetSdk).wait();
        }).future()();
    };
    AndroidProjectService.prototype.addLibrary = function (libraryPath) {
        var _this = this;
        return (function () {
            var name = path.basename(libraryPath);
            var targetLibPath = _this.getLibraryPath(name);
            var targetPath = path.dirname(targetLibPath);
            _this.$fs.ensureDirectoryExists(targetPath).wait();
            if (_this.$fs.exists(path.join(libraryPath, "project.properties")).wait()) {
                _this.parseProjectProperties(libraryPath, targetPath).wait();
            }
            shell.cp("-f", path.join(libraryPath, "*.jar"), targetPath);
            var projectLibsDir = path.join(_this.platformData.projectRoot, "libs");
            _this.$fs.ensureDirectoryExists(projectLibsDir).wait();
            shell.cp("-f", path.join(libraryPath, "*.jar"), projectLibsDir);
            var libProjProp = path.join(libraryPath, "project.properties");
            if (_this.$fs.exists(libProjProp).wait()) {
                _this.updateProjectReferences(_this.platformData.projectRoot, targetLibPath).wait();
            }
        }).future()();
    };
    AndroidProjectService.prototype.getFrameworkFilesExtensions = function () {
        return [".jar", ".dat"];
    };
    AndroidProjectService.prototype.prepareProject = function () {
        return (function () { }).future()();
    };
    AndroidProjectService.prototype.prepareAppResources = function (appResourcesDirectoryPath) {
        var _this = this;
        return (function () {
            var resourcesDirPath = path.join(appResourcesDirectoryPath, _this.platformData.normalizedPlatformName);
            var valuesDirRegExp = /^values/;
            var resourcesDirs = _this.$fs.readDirectory(resourcesDirPath).wait().filter(function (resDir) { return !resDir.match(valuesDirRegExp); });
            _.each(resourcesDirs, function (resourceDir) {
                _this.$fs.deleteDirectory(path.join(_this.platformData.appResourcesDestinationDirectoryPath, resourceDir)).wait();
            });
        }).future()();
    };
    AndroidProjectService.prototype.preparePluginNativeCode = function (pluginData) {
        var _this = this;
        return (function () {
            var pluginPlatformsFolderPath = _this.getPluginPlatformsFolderPath(pluginData, AndroidProjectService.ANDROID_PLATFORM_NAME);
            var libsFolderPath = path.join(pluginPlatformsFolderPath, AndroidProjectService.LIBS_FOLDER_NAME);
            if (_this.$fs.exists(libsFolderPath).wait()) {
                _this.addLibrary(libsFolderPath).wait();
            }
            var librarries = _this.getAllLibrariesForPlugin(pluginData).wait();
            _.each(librarries, function (libraryName) { return _this.addLibrary(path.join(pluginPlatformsFolderPath, libraryName)).wait(); });
        }).future()();
    };
    AndroidProjectService.prototype.removePluginNativeCode = function (pluginData) {
        var _this = this;
        return (function () {
            var projectPropertiesManager = _this.getProjectPropertiesManager(_this.platformData.projectRoot);
            var libraries = _this.getAllLibrariesForPlugin(pluginData).wait();
            _.each(libraries, function (libraryName) {
                var libraryPath = _this.getLibraryPath(libraryName);
                var libraryRelativePath = _this.getLibraryRelativePath(_this.platformData.projectRoot, libraryPath);
                projectPropertiesManager.removeProjectReference(libraryRelativePath).wait();
                _this.$fs.deleteDirectory(libraryPath).wait();
            });
        }).future()();
    };
    AndroidProjectService.prototype.getLibraryRelativePath = function (basePath, libraryPath) {
        return path.relative(basePath, libraryPath).split("\\").join("/");
    };
    AndroidProjectService.prototype.getLibraryPath = function (libraryName) {
        return path.join(this.$projectData.projectDir, "lib", this.platformData.normalizedPlatformName, libraryName);
    };
    AndroidProjectService.prototype.updateProjectReferences = function (projDir, libraryPath) {
        var relLibDir = this.getLibraryRelativePath(projDir, libraryPath);
        var projectPropertiesManager = this.getProjectPropertiesManager(projDir);
        return projectPropertiesManager.addProjectReference(relLibDir);
    };
    AndroidProjectService.prototype.getAllLibrariesForPlugin = function (pluginData) {
        var _this = this;
        return (function () {
            var filterCallback = function (fileName, pluginPlatformsFolderPath) { return fileName !== AndroidProjectService.LIBS_FOLDER_NAME && _this.$fs.exists(path.join(pluginPlatformsFolderPath, fileName, "project.properties")).wait(); };
            return _this.getAllNativeLibrariesForPlugin(pluginData, AndroidProjectService.ANDROID_PLATFORM_NAME, filterCallback).wait();
        }).future()();
    };
    AndroidProjectService.prototype.copy = function (projectRoot, frameworkDir, files, cpArg) {
        return (function () {
            var paths = files.split(' ').map(function (p) { return path.join(frameworkDir, p); });
            shell.cp(cpArg, paths, projectRoot);
        }).future()();
    };
    AndroidProjectService.prototype.spawn = function (command, args) {
        if (this.$hostInfo.isWindows) {
            args.unshift('/s', '/c', command);
            command = process.env.COMSPEC || 'cmd.exe';
        }
        return this.$childProcess.spawnFromEvent(command, args, "close", { stdio: "inherit" });
    };
    AndroidProjectService.prototype.getAntArgs = function (configuration, projectRoot) {
        var args = [configuration, "-f", path.join(projectRoot, "build.xml")];
        if (configuration === "release") {
            if (this.$options.keyStorePath) {
                args = args.concat(["-Dkey.store", this.$options.keyStorePath]);
            }
            if (this.$options.keyStorePassword) {
                args = args.concat(["-Dkey.store.password", this.$options.keyStorePassword]);
            }
            if (this.$options.keyStoreAlias) {
                args = args.concat(["-Dkey.alias", this.$options.keyStoreAlias]);
            }
            if (this.$options.keyStoreAliasPassword) {
                args = args.concat(["-Dkey.alias.password", this.$options.keyStoreAliasPassword]);
            }
        }
        args = args.concat(["-Dns.resources", path.join(__dirname, "../../resources/tools")]);
        return args;
    };
    AndroidProjectService.prototype.runAndroidUpdate = function (projectPath, targetApi) {
        var args = [
            "--path", projectPath,
            "--target", targetApi,
            "--name", this.$projectData.projectName
        ];
        return this.spawn("android", ['update', 'project'].concat(args));
    };
    AndroidProjectService.prototype.validatePackageName = function (packageName) {
        if (!/^[a-zA-Z]+(\.[a-zA-Z0-9][a-zA-Z0-9_]*)+$/.test(packageName)) {
            this.$errors.fail("Package name must look like: com.company.Name");
        }
        if (/\b[Cc]lass\b/.test(packageName)) {
            this.$errors.fail("class is a reserved word");
        }
    };
    AndroidProjectService.prototype.validateProjectName = function (projectName) {
        if (projectName === '') {
            this.$errors.fail("Project name cannot be empty");
        }
        if (/^[0-9]/.test(projectName)) {
            this.$errors.fail("Project name must not begin with a number");
        }
    };
    AndroidProjectService.prototype.getAndroidTarget = function (frameworkDir) {
        var _this = this;
        return (function () {
            var newTarget = _this.$options.sdk ? AndroidProjectService.ANDROID_TARGET_PREFIX + "-" + _this.$options.sdk : _this.getLatestValidAndroidTarget(frameworkDir).wait();
            if (!_.contains(_this.SUPPORTED_TARGETS, newTarget)) {
                var versionNumber = parseInt(_.last(newTarget.split("-")));
                if (versionNumber && (versionNumber < AndroidProjectService.MIN_SUPPORTED_VERSION)) {
                    _this.$errors.failWithoutHelp("The selected target SDK " + newTarget + " is not supported. You should target at least " + AndroidProjectService.MIN_SUPPORTED_VERSION + ".");
                }
                if (!_.contains(_this.getInstalledTargets().wait(), newTarget)) {
                    _this.$errors.failWithoutHelp(("You have selected to use " + newTarget + ", but it is not currently installed.") +
                        ' Run \"android\" from your command-line to install/update any missing SDKs or tools.');
                }
                _this.$logger.warn("The selected Android target '" + newTarget + "' is not verified as supported. Some functionality may not work as expected.");
            }
            return newTarget;
        }).future()();
    };
    AndroidProjectService.prototype.getLatestValidAndroidTarget = function (frameworkDir) {
        var _this = this;
        return (function () {
            var validTarget = _this.getTarget(frameworkDir).wait();
            var installedTargets = _this.getInstalledTargets().wait();
            var newTarget = _(_this.SUPPORTED_TARGETS).sort().findLast(function (supportedTarget) { return _.contains(installedTargets, supportedTarget); });
            if (!newTarget) {
                _this.$errors.failWithoutHelp(("Could not find supported Android target. Please install one of the following: " + _this.SUPPORTED_TARGETS.join(", ") + ".") +
                    " Make sure you have the latest Android tools installed as well." +
                    ' Run "android" from your command-line to install/update any missing SDKs or tools.');
            }
            return newTarget;
        }).future()();
    };
    AndroidProjectService.prototype.updateTarget = function (projectRoot, newTarget) {
        var _this = this;
        return (function () {
            var file = path.join(projectRoot, "project.properties");
            var editor = _this.$propertiesParser.createEditor(file).wait();
            editor.set("target", newTarget);
            var future = new Future();
            editor.save(function (err) {
                if (err) {
                    future.throw(err);
                }
                else {
                    _this.targetApi = null;
                    future.return();
                }
            });
            future.wait();
        }).future()();
    };
    AndroidProjectService.prototype.getInstalledTargets = function () {
        var _this = this;
        return (function () {
            if (!_this.installedTargetsCache) {
                _this.installedTargetsCache = [];
                var output = _this.$childProcess.exec('android list targets').wait();
                output.replace(/id: \d+ or "(.+)"/g, function (m, p1) { return (_this.installedTargetsCache.push(p1), m); });
            }
            return _this.installedTargetsCache;
        }).future()();
    };
    AndroidProjectService.prototype.getTarget = function (projectRoot) {
        var _this = this;
        return (function () {
            if (!_this.targetApi) {
                var projectPropertiesFilePath = path.join(projectRoot, "project.properties");
                if (_this.$fs.exists(projectPropertiesFilePath).wait()) {
                    var properties = _this.$propertiesParser.createEditor(projectPropertiesFilePath).wait();
                    _this.targetApi = properties.get("target");
                }
            }
            return _this.targetApi;
        }).future()();
    };
    AndroidProjectService.prototype.checkJava = function () {
        var _this = this;
        return (function () {
            try {
                var javaVersion = _this.$sysInfo.getSysInfo().javaVer;
                if (semver.lt(javaVersion, AndroidProjectService.MIN_JAVA_VERSION)) {
                    _this.$errors.failWithoutHelp("Your current java version is " + javaVersion + ". NativeScript CLI needs at least " + AndroidProjectService.MIN_JAVA_VERSION + " version to work correctly. Ensure that you have at least " + AndroidProjectService.MIN_JAVA_VERSION + " java version installed and try again.");
                }
            }
            catch (error) {
                _this.$errors.failWithoutHelp("Error executing command 'java'. Make sure you have java installed and added to your PATH.");
            }
        }).future()();
    };
    AndroidProjectService.prototype.checkAnt = function () {
        var _this = this;
        return (function () {
            try {
                _this.$childProcess.exec("ant -version").wait();
            }
            catch (error) {
                _this.$errors.fail("Error executing commands 'ant', make sure you have ant installed and added to your PATH.");
            }
        }).future()();
    };
    AndroidProjectService.prototype.checkAndroid = function () {
        var _this = this;
        return (function () {
            try {
                _this.$childProcess.exec('android list targets').wait();
            }
            catch (error) {
                if (error.match(/command\snot\sfound/)) {
                    _this.$errors.fail("The command \"android\" failed. Make sure you have the latest Android SDK installed, and the \"android\" command (inside the tools/ folder) is added to your path.");
                }
                else {
                    _this.$errors.fail("An error occurred while listing Android targets");
                }
            }
        }).future()();
    };
    AndroidProjectService.prototype.symlinkDirectory = function (directoryName, projectRoot, frameworkDir) {
        var _this = this;
        return (function () {
            _this.$fs.createDirectory(path.join(projectRoot, directoryName)).wait();
            var directoryContent = _this.$fs.readDirectory(path.join(frameworkDir, directoryName)).wait();
            _.each(directoryContent, function (file) {
                var sourceFilePath = path.join(frameworkDir, directoryName, file);
                var destinationFilePath = path.join(projectRoot, directoryName, file);
                if (_this.$fs.getFsStats(sourceFilePath).wait().isFile()) {
                    _this.$fs.symlink(sourceFilePath, destinationFilePath).wait();
                }
                else {
                    _this.$fs.symlink(sourceFilePath, destinationFilePath, "dir").wait();
                }
            });
        }).future()();
    };
    AndroidProjectService.MIN_SUPPORTED_VERSION = 17;
    AndroidProjectService.ANDROID_TARGET_PREFIX = "android";
    AndroidProjectService.RES_DIRNAME = "res";
    AndroidProjectService.VALUES_DIRNAME = "values";
    AndroidProjectService.VALUES_VERSION_DIRNAME_PREFIX = AndroidProjectService.VALUES_DIRNAME + "-v";
    AndroidProjectService.ANDROID_PLATFORM_NAME = "android";
    AndroidProjectService.LIBS_FOLDER_NAME = "libs";
    AndroidProjectService.MIN_JAVA_VERSION = "1.7.0";
    return AndroidProjectService;
})(projectServiceBaseLib.PlatformProjectServiceBase);
$injector.register("androidProjectService", AndroidProjectService);
