var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
var iOSProxyServices = require("./../common/mobile/ios/ios-proxy-services");
var net = require("net");
var ws = require("ws");
var stream = require("stream");
var path = require("path");
var Future = require("fibers/future");
var semver = require("semver");
var notification;
(function (notification_1) {
    function formatNotification(bundleId, notification) {
        return bundleId + ":NativeScript.Debug." + notification;
    }
    function waitForDebug(bundleId) {
        return formatNotification(bundleId, "WaitForDebugger");
    }
    notification_1.waitForDebug = waitForDebug;
    function attachRequest(bundleId) {
        return formatNotification(bundleId, "AttachRequest");
    }
    notification_1.attachRequest = attachRequest;
    function appLaunching(bundleId) {
        return formatNotification(bundleId, "AppLaunching");
    }
    notification_1.appLaunching = appLaunching;
    function readyForAttach(bundleId) {
        return formatNotification(bundleId, "ReadyForAttach");
    }
    notification_1.readyForAttach = readyForAttach;
    function attachAvailabilityQuery(bundleId) {
        return formatNotification(bundleId, "AttachAvailabilityQuery");
    }
    notification_1.attachAvailabilityQuery = attachAvailabilityQuery;
    function alreadyConnected(bundleId) {
        return formatNotification(bundleId, "AlreadyConnected");
    }
    notification_1.alreadyConnected = alreadyConnected;
    function attachAvailable(bundleId) {
        return formatNotification(bundleId, "AttachAvailable");
    }
    notification_1.attachAvailable = attachAvailable;
})(notification || (notification = {}));
var InspectorBackendPort = 18181;
function connectEventually(factory, handler) {
    function tryConnect() {
        var tryConnectAfterTimeout = setTimeout.bind(undefined, tryConnect, 1000);
        var socket = factory();
        socket.on("connect", function () {
            socket.removeListener("error", tryConnectAfterTimeout);
            handler(socket);
        });
        socket.on("error", tryConnectAfterTimeout);
    }
    tryConnect();
}
var IOSDebugService = (function () {
    function IOSDebugService($platformService, $iOSEmulatorServices, $devicesServices, $platformsData, $projectData, $childProcess, $logger, $fs, $errors, $injector, $npmInstallationManager, $options, $projectDataService, $utils) {
        this.$platformService = $platformService;
        this.$iOSEmulatorServices = $iOSEmulatorServices;
        this.$devicesServices = $devicesServices;
        this.$platformsData = $platformsData;
        this.$projectData = $projectData;
        this.$childProcess = $childProcess;
        this.$logger = $logger;
        this.$fs = $fs;
        this.$errors = $errors;
        this.$injector = $injector;
        this.$npmInstallationManager = $npmInstallationManager;
        this.$options = $options;
        this.$projectDataService = $projectDataService;
        this.$utils = $utils;
    }
    Object.defineProperty(IOSDebugService.prototype, "platform", {
        get: function () {
            return "ios";
        },
        enumerable: true,
        configurable: true
    });
    IOSDebugService.prototype.debug = function () {
        if (this.$options.debugBrk && this.$options.start) {
            this.$errors.failWithoutHelp("Expected exactly one of the --debug-brk or --start options.");
        }
        if (!this.$options.debugBrk && !this.$options.start) {
            this.$logger.warn("Neither --debug-brk nor --start option was specified. Defaulting to --debug-brk.");
            this.$options.debugBrk = true;
        }
        if (this.$options.emulator) {
            if (this.$options.debugBrk) {
                return this.emulatorDebugBrk();
            }
            else if (this.$options.start) {
                return this.emulatorStart();
            }
        }
        else {
            if (this.$options.debugBrk) {
                return this.deviceDebugBrk();
            }
            else if (this.$options.start) {
                return this.deviceStart();
            }
        }
        this.$errors.failWithoutHelp("Failed to select device or emulator to debug on.");
    };
    IOSDebugService.prototype.emulatorDebugBrk = function () {
        var _this = this;
        return (function () {
            var platformData = _this.$platformsData.getPlatformData(_this.platform);
            _this.$platformService.buildPlatform(_this.platform).wait();
            var emulatorPackage = _this.$platformService.getLatestApplicationPackageForEmulator(platformData).wait();
            _this.$iOSEmulatorServices.startEmulator(emulatorPackage.packageName, { args: "--nativescript-debug-brk" }).wait();
            createWebSocketProxy(_this.$logger, function (callback) { return connectEventually(function () { return net.connect(InspectorBackendPort); }, callback); });
            _this.executeOpenDebuggerClient().wait();
        }).future()();
    };
    IOSDebugService.prototype.emulatorStart = function () {
        var _this = this;
        return (function () {
            createWebSocketProxy(_this.$logger, function (callback) { return connectEventually(function () { return net.connect(InspectorBackendPort); }, callback); });
            _this.executeOpenDebuggerClient().wait();
            var projectId = _this.$projectData.projectId;
            var attachRequestMessage = notification.attachRequest(projectId);
            var iOSEmulator = _this.$iOSEmulatorServices;
            iOSEmulator.postDarwinNotification(attachRequestMessage).wait();
        }).future()();
    };
    IOSDebugService.prototype.deviceDebugBrk = function () {
        var _this = this;
        return (function () {
            _this.$devicesServices.initialize({ platform: _this.platform, deviceId: _this.$options.device }).wait();
            _this.$devicesServices.execute(function (device) { return (function () {
                var deploy = _this.$platformService.deployOnDevice(_this.platform);
                var iosDevice = device;
                var projectId = _this.$projectData.projectId;
                var npc = new iOSProxyServices.NotificationProxyClient(iosDevice, _this.$injector);
                try {
                    var timeout = _this.$utils.getMilliSecondsTimeout(IOSDebugService.TIMEOUT_SECONDS);
                    awaitNotification(npc, notification.appLaunching(projectId), timeout).wait();
                    process.nextTick(function () {
                        npc.postNotificationAndAttachForData(notification.waitForDebug(projectId));
                        npc.postNotificationAndAttachForData(notification.attachRequest(projectId));
                    });
                    awaitNotification(npc, notification.readyForAttach(projectId), _this.getReadyForAttachTimeout(timeout)).wait();
                }
                catch (e) {
                    _this.$logger.trace("Timeout error: " + e);
                    _this.$errors.failWithoutHelp("Timeout waiting for NativeScript debugger.");
                }
                createWebSocketProxy(_this.$logger, function (callback) { return connectEventually(function () { return iosDevice.connectToPort(InspectorBackendPort); }, callback); });
                _this.executeOpenDebuggerClient().wait();
                deploy.wait();
            }).future()(); }).wait();
        }).future()();
    };
    IOSDebugService.prototype.deviceStart = function () {
        var _this = this;
        return (function () {
            _this.$devicesServices.initialize({ platform: _this.platform, deviceId: _this.$options.device }).wait();
            _this.$devicesServices.execute(function (device) { return (function () {
                var iosDevice = device;
                var projectId = _this.$projectData.projectId;
                var npc = new iOSProxyServices.NotificationProxyClient(iosDevice, _this.$injector);
                var timeout = _this.getReadyForAttachTimeout();
                var _a = [
                    notification.alreadyConnected(projectId),
                    notification.readyForAttach(projectId),
                    notification.attachAvailable(projectId)
                ].map(function (notification) { return awaitNotification(npc, notification, timeout); }), alreadyConnected = _a[0], readyForAttach = _a[1], attachAvailable = _a[2];
                npc.postNotificationAndAttachForData(notification.attachAvailabilityQuery(projectId));
                var receivedNotification;
                try {
                    receivedNotification = whenAny(alreadyConnected, readyForAttach, attachAvailable).wait();
                }
                catch (e) {
                    _this.$errors.failWithoutHelp("The application " + projectId + " does not appear to be running on " + device.deviceInfo.displayName + " or is not built with debugging enabled.");
                }
                switch (receivedNotification) {
                    case alreadyConnected:
                        _this.$errors.failWithoutHelp("A debugger is already connected.");
                    case attachAvailable:
                        process.nextTick(function () { return npc.postNotificationAndAttachForData(notification.attachRequest(projectId)); });
                        try {
                            awaitNotification(npc, notification.readyForAttach(projectId), timeout).wait();
                        }
                        catch (e) {
                            _this.$errors.failWithoutHelp("The application " + projectId + " timed out when performing the NativeScript debugger handshake.");
                        }
                    case readyForAttach:
                        createWebSocketProxy(_this.$logger, function (callback) { return connectEventually(function () { return iosDevice.connectToPort(InspectorBackendPort); }, callback); });
                        _this.executeOpenDebuggerClient().wait();
                }
            }).future()(); }).wait();
        }).future()();
    };
    IOSDebugService.prototype.executeOpenDebuggerClient = function () {
        var _this = this;
        if (this.$options.client) {
            return this.openDebuggingClient();
        }
        else {
            return (function () {
                _this.$logger.info("Supressing debugging client.");
            }).future()();
        }
    };
    IOSDebugService.prototype.openDebuggingClient = function () {
        var _this = this;
        return (function () {
            _this.$projectDataService.initialize(_this.$projectData.projectDir);
            var platformData = _this.$platformsData.getPlatformData(_this.platform);
            var frameworkVersion = _this.$projectDataService.getValue(platformData.frameworkPackageName).wait().version;
            var inspectorPath = _this.getInspectorPath(frameworkVersion).wait();
            var inspectorSourceLocation = path.join(inspectorPath, "Safari/Main.html");
            var cmd = null;
            if (semver.lt(frameworkVersion, "1.2.0")) {
                cmd = "open -a Safari \"" + inspectorSourceLocation + "\"";
            }
            else {
                var inspectorApplicationPath = path.join(inspectorPath, "NativeScript Inspector.app");
                if (!_this.$fs.exists(inspectorApplicationPath).wait()) {
                    _this.$fs.unzip(path.join(inspectorPath, "NativeScript Inspector.zip"), inspectorPath).wait();
                }
                cmd = "open -a '" + inspectorApplicationPath + "' --args '" + inspectorSourceLocation + "' '" + _this.$projectData.projectName + "'";
            }
            _this.$childProcess.exec(cmd).wait();
        }).future()();
    };
    IOSDebugService.prototype.getInspectorPath = function (frameworkVersion) {
        var _this = this;
        return (function () {
            var tnsIosPackage = "";
            if (_this.$options.frameworkPath) {
                if (_this.$fs.getFsStats(_this.$options.frameworkPath).wait().isFile()) {
                    _this.$errors.failWithoutHelp("frameworkPath option must be path to directory which contains tns-ios framework");
                }
                tnsIosPackage = path.resolve(_this.$options.frameworkPath);
            }
            else {
                var platformData = _this.$platformsData.getPlatformData(_this.platform);
                tnsIosPackage = _this.$npmInstallationManager.install(platformData.frameworkPackageName, { version: frameworkVersion }).wait();
            }
            var inspectorPath = path.join(tnsIosPackage, "WebInspectorUI/");
            return inspectorPath;
        }).future()();
    };
    IOSDebugService.prototype.getReadyForAttachTimeout = function (timeoutInMilliseconds) {
        var timeout = timeoutInMilliseconds || this.$utils.getMilliSecondsTimeout(IOSDebugService.TIMEOUT_SECONDS);
        var readyForAttachTimeout = timeout / 10;
        var defaultReadyForAttachTimeout = 5000;
        return readyForAttachTimeout > defaultReadyForAttachTimeout ? readyForAttachTimeout : defaultReadyForAttachTimeout;
    };
    IOSDebugService.TIMEOUT_SECONDS = 90;
    return IOSDebugService;
})();
$injector.register("iOSDebugService", IOSDebugService);
function createWebSocketProxy($logger, socketFactory) {
    var localPort = 8080;
    $logger.info("\nSetting up debugger proxy...\nPress Ctrl + C to terminate, or disconnect.\n");
    var server = ws.createServer({
        port: localPort,
        verifyClient: function (info, callback) {
            $logger.info("Frontend client connected.");
            socketFactory(function (socket) {
                $logger.info("Backend socket created.");
                info.req["__deviceSocket"] = socket;
                callback(true);
            });
        }
    });
    server.on("connection", function (webSocket) {
        var deviceSocket = webSocket.upgradeReq["__deviceSocket"];
        var packets = new PacketStream();
        deviceSocket.pipe(packets);
        packets.on("data", function (buffer) {
            webSocket.send(buffer.toString("utf16le"));
        });
        webSocket.on("message", function (message, flags) {
            var length = Buffer.byteLength(message, "utf16le");
            var payload = new Buffer(length + 4);
            payload.writeInt32BE(length, 0);
            payload.write(message, 4, length, "utf16le");
            deviceSocket.write(payload);
        });
        deviceSocket.on("end", function () {
            $logger.info("Backend socket closed!");
            process.exit(0);
        });
        webSocket.on("close", function () {
            $logger.info('Frontend socket closed!');
            process.exit(0);
        });
    });
    $logger.info("Opened localhost " + localPort);
    return server;
}
function awaitNotification(npc, notification, timeout) {
    var future = new Future();
    var timeoutObject = setTimeout(function () {
        detachObserver();
        future.throw(new Error("Timeout receiving " + notification + " notification."));
    }, timeout);
    function notificationObserver(notification) {
        clearTimeout(timeoutObject);
        detachObserver();
        future.return(notification);
    }
    function detachObserver() {
        process.nextTick(function () { return npc.removeObserver(notification, notificationObserver); });
    }
    npc.addObserver(notification, notificationObserver);
    return future;
}
function whenAny() {
    var futures = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        futures[_i - 0] = arguments[_i];
    }
    var resultFuture = new Future();
    var futuresLeft = futures.length;
    for (var _a = 0; _a < futures.length; _a++) {
        var future = futures[_a];
        var futureLocal = future;
        future.resolve(function (error, result) {
            futuresLeft--;
            if (!resultFuture.isResolved()) {
                if (typeof error === "undefined") {
                    resultFuture.return(futureLocal);
                }
                else if (futuresLeft == 0) {
                    resultFuture.throw(new Error("None of the futures succeeded."));
                }
            }
        });
    }
    return resultFuture;
}
var PacketStream = (function (_super) {
    __extends(PacketStream, _super);
    function PacketStream(opts) {
        _super.call(this, opts);
    }
    PacketStream.prototype._transform = function (packet, encoding, done) {
        while (packet.length > 0) {
            if (!this.buffer) {
                var length = packet.readInt32BE(0);
                this.buffer = new Buffer(length);
                this.offset = 0;
                packet = packet.slice(4);
            }
            packet.copy(this.buffer, this.offset);
            var copied = Math.min(this.buffer.length - this.offset, packet.length);
            this.offset += copied;
            packet = packet.slice(copied);
            if (this.offset === this.buffer.length) {
                this.push(this.buffer);
                this.buffer = undefined;
            }
        }
        done();
    };
    return PacketStream;
})(stream.Transform);
