///<reference path="../.d.ts"/>
"use strict";
var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
var path = require("path");
var shell = require("shelljs");
var util = require("util");
var xcode = require("xcode");
var constants = require("./../constants");
var helpers = require("./../common/helpers");
var projectServiceBaseLib = require("./platform-project-service-base");
var IOSProjectService = (function (_super) {
    __extends(IOSProjectService, _super);
    function IOSProjectService($projectData, $fs, $childProcess, $errors, $logger, $iOSEmulatorServices, $options, $injector) {
        _super.call(this, $fs);
        this.$projectData = $projectData;
        this.$childProcess = $childProcess;
        this.$errors = $errors;
        this.$logger = $logger;
        this.$iOSEmulatorServices = $iOSEmulatorServices;
        this.$options = $options;
        this.$injector = $injector;
    }
    Object.defineProperty(IOSProjectService.prototype, "$npmInstallationManager", {
        get: function () {
            return this.$injector.resolve("npmInstallationManager");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IOSProjectService.prototype, "platformData", {
        get: function () {
            var projectRoot = path.join(this.$projectData.platformsDir, "ios");
            return {
                frameworkPackageName: "tns-ios",
                normalizedPlatformName: "iOS",
                appDestinationDirectoryPath: path.join(projectRoot, this.$projectData.projectName),
                appResourcesDestinationDirectoryPath: path.join(projectRoot, this.$projectData.projectName, "Resources", "icons"),
                platformProjectService: this,
                emulatorServices: this.$iOSEmulatorServices,
                projectRoot: projectRoot,
                deviceBuildOutputPath: path.join(projectRoot, "build", "device"),
                emulatorBuildOutputPath: path.join(projectRoot, "build", "emulator"),
                validPackageNamesForDevice: [
                    this.$projectData.projectName + ".ipa"
                ],
                validPackageNamesForEmulator: [
                    this.$projectData.projectName + ".app"
                ],
                frameworkFilesExtensions: [".a", ".framework", ".bin"],
                frameworkDirectoriesExtensions: [".framework"],
                frameworkDirectoriesNames: ["Metadata"],
                targetedOS: ['darwin'],
                configurationFileName: this.$projectData.projectName + "-Info.plist",
                configurationFilePath: path.join(projectRoot, this.$projectData.projectName, this.$projectData.projectName + "-Info.plist")
            };
        },
        enumerable: true,
        configurable: true
    });
    IOSProjectService.prototype.validate = function () {
        var _this = this;
        return (function () {
            try {
                _this.$childProcess.exec("which xcodebuild").wait();
            }
            catch (error) {
                _this.$errors.fail("Xcode is not installed. Make sure you have Xcode installed and added to your PATH");
            }
            var xcodeBuildVersion = _this.$childProcess.exec("xcodebuild -version | head -n 1 | sed -e 's/Xcode //'").wait();
            var splitedXcodeBuildVersion = xcodeBuildVersion.split(".");
            if (splitedXcodeBuildVersion.length === 3) {
                xcodeBuildVersion = util.format("%s.%s", splitedXcodeBuildVersion[0], splitedXcodeBuildVersion[1]);
            }
            if (helpers.versionCompare(xcodeBuildVersion, IOSProjectService.XCODEBUILD_MIN_VERSION) < 0) {
                _this.$errors.fail("NativeScript can only run in Xcode version %s or greater", IOSProjectService.XCODEBUILD_MIN_VERSION);
            }
        }).future()();
    };
    IOSProjectService.prototype.createProject = function (projectRoot, frameworkDir) {
        var _this = this;
        return (function () {
            if (_this.$options.symlink) {
                _this.$fs.ensureDirectoryExists(path.join(projectRoot, IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER)).wait();
                var xcodeProjectName = util.format("%s.xcodeproj", IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER);
                shell.cp("-R", path.join(frameworkDir, IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER, "*"), path.join(projectRoot, IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER));
                shell.cp("-R", path.join(frameworkDir, xcodeProjectName), projectRoot);
                var directoryContent = _this.$fs.readDirectory(frameworkDir).wait();
                var frameworkFiles = _.difference(directoryContent, [IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER, xcodeProjectName]);
                _.each(frameworkFiles, function (file) {
                    _this.$fs.symlink(path.join(frameworkDir, file), path.join(projectRoot, file)).wait();
                });
            }
            else {
                shell.cp("-R", path.join(frameworkDir, "*"), projectRoot);
            }
        }).future()();
    };
    IOSProjectService.prototype.interpolateData = function (projectRoot) {
        var _this = this;
        return (function () {
            var infoPlistFilePath = path.join(projectRoot, IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER, util.format("%s-%s", IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER, "Info.plist"));
            shell.sed('-i', "__CFBUNDLEIDENTIFIER__", _this.$projectData.projectId, infoPlistFilePath);
            _this.replaceFileName("-Info.plist", path.join(projectRoot, IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER)).wait();
            _this.replaceFileName("-Prefix.pch", path.join(projectRoot, IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER)).wait();
            _this.replaceFileName(IOSProjectService.XCODE_PROJECT_EXT_NAME, projectRoot).wait();
            var pbxprojFilePath = path.join(projectRoot, _this.$projectData.projectName + IOSProjectService.XCODE_PROJECT_EXT_NAME, "project.pbxproj");
            _this.replaceFileContent(pbxprojFilePath).wait();
            var mainFilePath = path.join(projectRoot, IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER, "main.m");
            _this.replaceFileContent(mainFilePath).wait();
        }).future()();
    };
    IOSProjectService.prototype.afterCreateProject = function (projectRoot) {
        var _this = this;
        return (function () {
            _this.$fs.rename(path.join(projectRoot, IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER), path.join(projectRoot, _this.$projectData.projectName)).wait();
        }).future()();
    };
    IOSProjectService.prototype.buildProject = function (projectRoot) {
        var _this = this;
        return (function () {
            var basicArgs = [
                "-project", path.join(projectRoot, _this.$projectData.projectName + ".xcodeproj"),
                "-target", _this.$projectData.projectName,
                "-configuration", _this.$options.release ? "Release" : "Debug",
                "build",
                'SHARED_PRECOMPS_DIR=' + path.join(projectRoot, 'build', 'sharedpch')
            ];
            var args = [];
            if (_this.$options.forDevice) {
                args = basicArgs.concat([
                    "-xcconfig", path.join(projectRoot, _this.$projectData.projectName, "build.xcconfig"),
                    "-sdk", "iphoneos",
                    'ARCHS=armv7 arm64',
                    'VALID_ARCHS=armv7 arm64',
                    "CONFIGURATION_BUILD_DIR=" + path.join(projectRoot, "build", "device")
                ]);
            }
            else {
                args = basicArgs.concat([
                    "-sdk", "iphonesimulator",
                    "-arch", "i386",
                    "VALID_ARCHS=\"i386\"",
                    "CONFIGURATION_BUILD_DIR=" + path.join(projectRoot, "build", "emulator")
                ]);
            }
            _this.$childProcess.spawnFromEvent("xcodebuild", args, "exit", { cwd: _this.$options, stdio: 'inherit' }).wait();
            if (_this.$options.forDevice) {
                var buildOutputPath = path.join(projectRoot, "build", "device");
                var xcrunArgs = [
                    "-sdk", "iphoneos",
                    "PackageApplication",
                    "-v", path.join(buildOutputPath, _this.$projectData.projectName + ".app"),
                    "-o", path.join(buildOutputPath, _this.$projectData.projectName + ".ipa")
                ];
                _this.$childProcess.spawnFromEvent("xcrun", xcrunArgs, "exit", { cwd: _this.$options, stdio: 'inherit' }).wait();
            }
        }).future()();
    };
    IOSProjectService.prototype.isPlatformPrepared = function (projectRoot) {
        return this.$fs.exists(path.join(projectRoot, this.$projectData.projectName, constants.APP_FOLDER_NAME));
    };
    IOSProjectService.prototype.addLibrary = function (libraryPath) {
        var _this = this;
        return (function () {
            _this.validateDynamicFramework(libraryPath).wait();
            var umbrellaHeader = _this.getUmbrellaHeaderFromDynamicFramework(libraryPath).wait();
            var frameworkName = path.basename(libraryPath, path.extname(libraryPath));
            var targetPath = path.join("lib", _this.platformData.normalizedPlatformName);
            var fullTargetPath = path.join(_this.$projectData.projectDir, targetPath);
            _this.$fs.ensureDirectoryExists(fullTargetPath).wait();
            shell.cp("-R", libraryPath, fullTargetPath);
            var project = _this.createPbxProj();
            var frameworkPath = _this.getFrameworkRelativePath(libraryPath);
            project.addFramework(frameworkPath, { customFramework: true, embed: true });
            project.updateBuildProperty("IPHONEOS_DEPLOYMENT_TARGET", "8.0");
            _this.savePbxProj(project).wait();
            _this.$logger.info("The iOS Deployment Target is now 8.0 in order to support Cocoa Touch Frameworks.");
        }).future()();
    };
    IOSProjectService.prototype.canUpdatePlatform = function (currentVersion, newVersion) {
        var _this = this;
        return (function () {
            var currentXcodeProjectFile = _this.buildPathToXcodeProjectFile(currentVersion);
            var currentXcodeProjectFileContent = _this.$fs.readFile(currentXcodeProjectFile).wait();
            var newXcodeProjectFile = _this.buildPathToXcodeProjectFile(newVersion);
            var newXcodeProjectFileContent = _this.$fs.readFile(newXcodeProjectFile).wait();
            return currentXcodeProjectFileContent === newXcodeProjectFileContent;
        }).future()();
    };
    IOSProjectService.prototype.updatePlatform = function (currentVersion, newVersion) {
        var _this = this;
        return (function () {
            var sourceFile = path.join(_this.platformData.projectRoot, util.format("%s.xcodeproj", _this.$projectData.projectName));
            var destinationFile = path.join(_this.$options.profileDir, "xcodeproj");
            _this.$fs.deleteDirectory(destinationFile).wait();
            shell.cp("-R", path.join(sourceFile, "*"), destinationFile);
            _this.$logger.info("Backup file %s at location %s", sourceFile, destinationFile);
            _this.$fs.deleteDirectory(path.join(_this.platformData.projectRoot, util.format("%s.xcodeproj", _this.$projectData.projectName))).wait();
            var cachedPackagePath = path.join(_this.$npmInstallationManager.getCachedPackagePath(_this.platformData.frameworkPackageName, newVersion), constants.PROJECT_FRAMEWORK_FOLDER_NAME, util.format("%s.xcodeproj", IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER));
            shell.cp("-R", path.join(cachedPackagePath, "*"), path.join(_this.platformData.projectRoot, util.format("%s.xcodeproj", _this.$projectData.projectName)));
            _this.$logger.info("Copied from %s at %s.", cachedPackagePath, _this.platformData.projectRoot);
            var pbxprojFilePath = path.join(_this.platformData.projectRoot, _this.$projectData.projectName + IOSProjectService.XCODE_PROJECT_EXT_NAME, "project.pbxproj");
            _this.replaceFileContent(pbxprojFilePath).wait();
        }).future()();
    };
    IOSProjectService.prototype.prepareProject = function () {
        var _this = this;
        return (function () {
            var project = _this.createPbxProj();
            var resources = project.pbxGroupByName("Resources");
            if (resources) {
                var references = project.pbxFileReferenceSection();
                var xcodeProjectImages = _.map(resources.children, function (resource) { return _this.replace(references[resource.value].name); });
                _this.$logger.trace("Images from Xcode project");
                _this.$logger.trace(xcodeProjectImages);
                var appResourcesImages = _this.$fs.readDirectory(_this.platformData.appResourcesDestinationDirectoryPath).wait();
                _this.$logger.trace("Current images from App_Resources");
                _this.$logger.trace(appResourcesImages);
                var imagesToAdd = _.difference(appResourcesImages, xcodeProjectImages);
                _this.$logger.trace("New images to add into xcode project: " + imagesToAdd.join(", "));
                _.each(imagesToAdd, function (image) { return project.addResourceFile(path.relative(_this.platformData.projectRoot, path.join(_this.platformData.appResourcesDestinationDirectoryPath, image))); });
                var imagesToRemove = _.difference(xcodeProjectImages, appResourcesImages);
                _this.$logger.trace("Images to remove from xcode project: " + imagesToRemove.join(", "));
                _.each(imagesToRemove, function (image) { return project.removeResourceFile(path.join(_this.platformData.appResourcesDestinationDirectoryPath, image)); });
                _this.savePbxProj(project).wait();
            }
        }).future()();
    };
    IOSProjectService.prototype.prepareAppResources = function (appResourcesDirectoryPath) {
        return this.$fs.deleteDirectory(this.platformData.appResourcesDestinationDirectoryPath);
    };
    IOSProjectService.prototype.replace = function (name) {
        if (_.startsWith(name, '"')) {
            name = name.substr(1, name.length - 2);
        }
        return name.replace(/\\\"/g, "\"");
    };
    IOSProjectService.prototype.getFrameworkRelativePath = function (libraryPath) {
        var frameworkName = path.basename(libraryPath, path.extname(libraryPath));
        var targetPath = path.join("lib", this.platformData.normalizedPlatformName);
        var frameworkPath = path.relative("platforms/ios", path.join(targetPath, frameworkName + ".framework"));
        return frameworkPath;
    };
    Object.defineProperty(IOSProjectService.prototype, "pbxProjPath", {
        get: function () {
            return path.join(this.platformData.projectRoot, this.$projectData.projectName + ".xcodeproj", "project.pbxproj");
        },
        enumerable: true,
        configurable: true
    });
    IOSProjectService.prototype.createPbxProj = function () {
        var project = new xcode.project(this.pbxProjPath);
        project.parseSync();
        return project;
    };
    IOSProjectService.prototype.savePbxProj = function (project) {
        return this.$fs.writeFile(this.pbxProjPath, project.writeSync());
    };
    IOSProjectService.prototype.preparePluginNativeCode = function (pluginData) {
        var _this = this;
        return (function () {
            var pluginPlatformsFolderPath = pluginData.pluginPlatformsFolderPath(IOSProjectService.IOS_PLATFORM_NAME);
            _.each(_this.getAllDynamicFrameworksForPlugin(pluginData).wait(), function (fileName) { return _this.addLibrary(path.join(pluginPlatformsFolderPath, fileName)).wait(); });
        }).future()();
    };
    IOSProjectService.prototype.removePluginNativeCode = function (pluginData) {
        var _this = this;
        return (function () {
            var pluginPlatformsFolderPath = pluginData.pluginPlatformsFolderPath(IOSProjectService.IOS_PLATFORM_NAME);
            var project = _this.createPbxProj();
            _.each(_this.getAllDynamicFrameworksForPlugin(pluginData).wait(), function (fileName) {
                var fullFrameworkPath = path.join(pluginPlatformsFolderPath, fileName);
                var relativeFrameworkPath = _this.getFrameworkRelativePath(fullFrameworkPath);
                project.removeFramework(relativeFrameworkPath, { customFramework: true, embed: true });
            });
            _this.savePbxProj(project).wait();
        }).future()();
    };
    IOSProjectService.prototype.getAllDynamicFrameworksForPlugin = function (pluginData) {
        var filterCallback = function (fileName, pluginPlatformsFolderPath) { return path.extname(fileName) === ".framework"; };
        return this.getAllNativeLibrariesForPlugin(pluginData, IOSProjectService.IOS_PLATFORM_NAME, filterCallback);
    };
    IOSProjectService.prototype.buildPathToXcodeProjectFile = function (version) {
        return path.join(this.$npmInstallationManager.getCachedPackagePath(this.platformData.frameworkPackageName, version), constants.PROJECT_FRAMEWORK_FOLDER_NAME, util.format("%s.xcodeproj", IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER), "project.pbxproj");
    };
    IOSProjectService.prototype.validateDynamicFramework = function (libraryPath) {
        var _this = this;
        return (function () {
            var infoPlistPath = path.join(libraryPath, "Info.plist");
            if (!_this.$fs.exists(infoPlistPath).wait()) {
                _this.$errors.failWithoutHelp("The bundle at %s does not contain an Info.plist file.", libraryPath);
            }
            var packageType = _this.$childProcess.exec("/usr/libexec/PlistBuddy -c \"Print :CFBundlePackageType\" \"" + infoPlistPath + "\"").wait().trim();
            if (packageType !== "FMWK") {
                _this.$errors.failWithoutHelp("The bundle at %s does not appear to be a dynamic framework.", libraryPath);
            }
        }).future()();
    };
    IOSProjectService.prototype.getUmbrellaHeaderFromDynamicFramework = function (libraryPath) {
        var _this = this;
        return (function () {
            var modulemapPath = path.join(libraryPath, "Modules", "module.modulemap");
            if (!_this.$fs.exists(modulemapPath).wait()) {
                _this.$errors.failWithoutHelp("The framework at %s does not contain a module.modulemap file.", modulemapPath);
            }
            var modulemap = _this.$fs.readText(modulemapPath).wait();
            var umbrellaRegex = /umbrella header "(.+\.h)"/g;
            var match = umbrellaRegex.exec(modulemap);
            if (!match) {
                _this.$errors.failWithoutHelp("The modulemap at %s does not specify an umbrella header.", modulemapPath);
            }
            return match[1];
        }).future()();
    };
    IOSProjectService.prototype.replaceFileContent = function (file) {
        var _this = this;
        return (function () {
            var fileContent = _this.$fs.readText(file).wait();
            var replacedContent = helpers.stringReplaceAll(fileContent, IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER, _this.$projectData.projectName);
            _this.$fs.writeFile(file, replacedContent).wait();
        }).future()();
    };
    IOSProjectService.prototype.replaceFileName = function (fileNamePart, fileRootLocation) {
        var _this = this;
        return (function () {
            var oldFileName = IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER + fileNamePart;
            var newFileName = _this.$projectData.projectName + fileNamePart;
            _this.$fs.rename(path.join(fileRootLocation, oldFileName), path.join(fileRootLocation, newFileName)).wait();
        }).future()();
    };
    IOSProjectService.XCODE_PROJECT_EXT_NAME = ".xcodeproj";
    IOSProjectService.XCODEBUILD_MIN_VERSION = "6.0";
    IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER = "__PROJECT_NAME__";
    IOSProjectService.IOS_PLATFORM_NAME = "ios";
    return IOSProjectService;
})(projectServiceBaseLib.PlatformProjectServiceBase);
$injector.register("iOSProjectService", IOSProjectService);
